import unittest

import src.hkkang_utils.string as string_utils


class Test_string_utils(unittest.TestCase):
    def __init__(self, *args, **kwargs):
        super(Test_string_utils, self).__init__(*args, **kwargs)
    
    def test_multi_space_to_single_space(self):
        pass
    
    def test_is_int(self):
        input_str1 = "123"
        input_str2 = "123.0"
        input_str3 = "123.1"
        input_str4  = "abc1"
        input_str5  = "1abc"
        input_str6  = "1abc1"
        self.assertTrue(string_utils.is_int(input_str1))
        self.assertFalse(string_utils.is_int(input_str2))
        self.assertFalse(string_utils.is_int(input_str3))
        self.assertFalse(string_utils.is_int(input_str4))
        self.assertFalse(string_utils.is_int(input_str5))
        self.assertFalse(string_utils.is_int(input_str6))
        
    
    def test_is_number(self):
        input_str1 = "123"
        input_str2 = "123.0"
        input_str3 = "123.1"
        input_str4  = "abc1"
        input_str5  = "1abc"
        input_str6  = "1abc1"
        self.assertTrue(string_utils.is_number(input_str1), f"{input_str1} is int")
        self.assertTrue(string_utils.is_number(input_str2), f"{input_str2} is int")
        self.assertTrue(string_utils.is_number(input_str3), f"{input_str3} is int")
        self.assertFalse(string_utils.is_number(input_str4), f"{input_str4} is not int")
        self.assertFalse(string_utils.is_number(input_str5), f"{input_str5} is not int")
        self.assertFalse(string_utils.is_number(input_str6), f"{input_str6} is not int")

    def test_remove_substring(self):
        input_str = "abcdefg"
        self.assertTrue(string_utils.remove_substring(input_str, "ac") == "abcdefg")
        self.assertTrue(string_utils.remove_substring(input_str, "de") == "abcfg")

    def test_remove_punctuation(self):
        input_str = "a,b.c;d:e?fg"
        self.assertTrue(string_utils.remove_punctuation(input_str) == "abcdefg")

if __name__ == "__main__":
    unittest.main()



import threading

def shorten_string(text: str) -> str:
    assert isinstance(text, str), f"text must be str type, but {type(text)} is given."
    if len(text) > 10:
        return text[:10] + "..."
    return text
    

class Thread(threading.Thread):
    """Please use start method to start thread. start method will call run method.

    example:
        # Initialize threads
        thread1 = concurrent_utils.Thread(1, count_million_and_print_name, 1, {"name":"name_1"})
        thread2 = concurrent_utils.Thread(2, count_million_and_print_name, 2, {"name":"name_2"})
        
        # Start threads
        thread1.start()
        thread2.start()

        # Wait for threads to finish (this is optional: accessing result will call join method anyway)
        thread1.join()
        thread2.join()
        
        # Get results
        result1 = thread1.result
        result2 = thread2.result
    """
    # Static variable
    cnt = 0
    def __init__(self, target, args=None, kwargs=None):
        super().__init__()
        Thread.cnt += 1
        self.threadID = Thread.cnt
        self.func = target
        self.args = self._parse_args(args)
        self.kwargs = self._parse_kwargs(kwargs)
        self._result = None
    
    @property
    def result(self):
        if self.is_alive():
            self.join()
        return self._result
    
    def run(self):
        post_fix = f"Thread {self.threadID}: {self.func.__name__}(args={shorten_string(str(self.args))}, kwargs={shorten_string(str(self.kwargs))})"
        print(f"Starting {post_fix}")
        self._result = self.func(*self.args, **self.kwargs)
        print(f"Exiting {post_fix}")
    
    def _parse_args(self, args):
        if args is None:
            return ()
        elif isinstance(args, tuple):
            return args
        elif isinstance(args, list):
            return tuple(args)
        else:
            return (args,)
        
    def _parse_kwargs(self, kwargs):
        if kwargs is None:
            return {}
        elif isinstance(kwargs, dict):
            return kwargs
        else:
            raise ValueError(f"kwargs must be dict type, but {type(kwargs)} is given.")


if __name__ == "__main__":
    pass